@@ -20,11 +20,8 @@ typedef struct ncinput {
   bool ctrl;       // Was Ctrl held during the event?
   uint64_t seqnum; // Monotonically increasing input event counter
 } ncinput;
-bool nckey_mouse_p(char32_t r);
 // sigset_t differs from system to system, annoying
 // char32_t notcurses_getc(struct notcurses* n, const struct timespec* ts, sigset_t* sigmask, ncinput* ni);
-char32_t notcurses_getc_nblock(struct notcurses* n, ncinput* ni);
-char32_t notcurses_getc_blocking(struct notcurses* n, ncinput* ni);
 int notcurses_inputready_fd(struct notcurses* n);
 typedef struct cell {
   uint32_t gcluster;          // 4B → 4B
@@ -59,12 +56,15 @@ int notcurses_render(struct notcurses*);
 int notcurses_render_to_file(struct notcurses* nc, FILE* fp);
 struct ncplane* notcurses_stdplane(struct notcurses*);
 const struct ncplane* notcurses_stdplane_const(const struct notcurses* nc);
-void ncplane_set_channels(struct ncplane* nc, uint64_t channels);
+void ncplane_set_channels(struct ncplane* n, uint64_t channels);
+uint64_t ncplane_set_fchannel(struct ncplane* n, uint32_t channel);
+uint64_t ncplane_set_bchannel(struct ncplane* n, uint32_t channel);
 void ncplane_set_attr(struct ncplane* n, unsigned stylebits);
 int ncplane_set_base_cell(struct ncplane* ncp, const cell* c);
-int ncplane_set_base(struct ncplane* ncp, const char* egc, uint32_t attrword, uint64_t channels);
+int ncplane_set_base(struct ncplane* ncp, const char* egc, uint32_t styles, uint64_t channels);
 int ncplane_base(struct ncplane* ncp, cell* c);
 struct ncplane* notcurses_top(struct notcurses* n);
+struct ncplane* notcurses_bottom(struct notcurses* n);
 void notcurses_drop_planes(struct notcurses* nc);
 int notcurses_refresh(struct notcurses* n, int* restrict y, int* restrict x);
 struct ncplane* ncplane_reparent(struct ncplane* n, struct ncplane* newparent);
@@ -91,7 +91,8 @@ bool notcurses_cansixel(const struct notcurses* nc);
 int notcurses_mouse_enable(struct notcurses* n);
 int notcurses_mouse_disable(struct notcurses* n);
 int ncplane_destroy(struct ncplane* ncp);
-int ncplane_mergedown(struct ncplane* restrict src, struct ncplane* restrict dst);
+int ncplane_mergedown(const struct ncplane* src, struct ncplane* dst, int begsrcy, int begsrcx, int leny, int lenx, int dsty, int dstx);
+int ncplane_mergedown_simple(const struct ncplane* restrict src, struct ncplane* restrict dst);
 void ncplane_erase(struct ncplane* n);
 int ncplane_cursor_move_yx(struct ncplane* n, int y, int x);
 void ncplane_cursor_yx(struct ncplane* n, int* y, int* x);
@@ -104,11 +105,10 @@ void ncplane_move_bottom(struct ncplane* n);
 int ncplane_move_below(struct ncplane* restrict n, struct ncplane* restrict below);
 int ncplane_move_above(struct ncplane* restrict n, struct ncplane* restrict above);
 struct ncplane* ncplane_below(struct ncplane* n);
+struct ncplane* ncplane_above(struct ncplane* n);
 char* notcurses_at_yx(struct notcurses* nc, int yoff, int xoff, uint16_t* stylemask, uint64_t* channels);
 char* ncplane_at_cursor(struct ncplane* n, uint16_t* stylemask, uint64_t* channels);
-int ncplane_at_cursor_cell(struct ncplane* n, cell* c);
 char* ncplane_at_yx(const struct ncplane* n, int y, int x, uint16_t* stylemask, uint64_t* channels);
-int ncplane_at_yx_cell(struct ncplane* n, int y, int x, cell* c);
 typedef enum {
   NCBLIT_1x1,     // full block                █
   NCBLIT_2x1,     // full/(upper|left) blocks  ▄█
@@ -127,16 +127,9 @@ void* ncplane_set_userptr(struct ncplane* n, void* opaque);
 void* ncplane_userptr(struct ncplane* n);
 int ncplane_resize(struct ncplane* n, int keepy, int keepx, int keepleny,
                        int keeplenx, int yoff, int xoff, int ylen, int xlen);
-uint64_t ncplane_channels(struct ncplane* n);
-uint16_t ncplane_attr(struct ncplane* n);
-unsigned ncplane_bchannel(struct ncplane* nc);
-unsigned ncplane_fchannel(struct ncplane* nc);
-unsigned ncplane_fg(struct ncplane* nc);
-unsigned ncplane_bg(struct ncplane* nc);
-unsigned ncplane_fg_alpha(struct ncplane* nc);
-unsigned ncplane_bg_alpha(struct ncplane* nc);
-unsigned ncplane_fg_rgb(struct ncplane* n, unsigned* r, unsigned* g, unsigned* b);
-unsigned ncplane_bg_rgb(struct ncplane* n, unsigned* r, unsigned* g, unsigned* b);
+uint64_t ncplane_channels(const struct ncplane* n);
+uint16_t ncplane_attr(const struct ncplane* n);
+unsigned ncplane_styles(const struct ncplane* n);
 int ncplane_set_fg_rgb(struct ncplane* n, int r, int g, int b);
 int ncplane_set_bg_rgb(struct ncplane* n, int r, int g, int b);
 void ncplane_set_fg_rgb_clipped(struct ncplane* n, int r, int g, int b);
@@ -180,66 +173,21 @@ typedef int (*fadecb)(struct notcurses* nc, struct ncplane* ncp, const struct ti
 int ncplane_fadeout(struct ncplane* n, const struct timespec* ts, fadecb fader, void* curry);
 int ncplane_fadein(struct ncplane* n, const struct timespec* ts, fadecb fader, void* curry);
 int ncplane_pulse(struct ncplane* n, const struct timespec* ts, fadecb fader, void* curry);
-int ncplane_putwc_yx(struct ncplane* n, int y, int x, wchar_t w);
-int ncplane_putwc(struct ncplane* n, wchar_t w);
 int ncplane_putegc_yx(struct ncplane* n, int y, int x, const char* gclust, int* sbytes);
 int ncplane_putstr_aligned(struct ncplane* n, int y, ncalign_e align, const char* s);
 int ncplane_putstr_stainable(struct ncplane* n, const char* s);
 struct ncplane* ncplane_dup(const struct ncplane* n, void* opaque);
-void cell_init(cell* c);
 int cell_load(struct ncplane* n, cell* c, const char* gcluster);
-int cell_prime(struct ncplane* n, cell* c, const char* gcluster, uint32_t attr, uint64_t channels);
 int cell_duplicate(struct ncplane* n, cell* targ, const cell* c);
 void cell_release(struct ncplane* n, cell* c);
-void cell_styles_set(cell* c, unsigned stylebits);
-unsigned cell_styles(const cell* c);
-void cell_styles_on(cell* c, unsigned stylebits);
-void cell_styles_off(cell* c, unsigned stylebits);
-void cell_set_fg_default(cell* c);
-void cell_set_bg_default(cell* c);
-int cell_set_fg_alpha(cell* c, unsigned alpha);
-int cell_set_bg_alpha(cell* c, unsigned alpha);
-bool cell_double_wide_p(const cell* c);
-bool cell_simple_p(const cell* c);
 const char* cell_extended_gcluster(const struct ncplane* n, const cell* c);
-int cell_load_simple(struct ncplane* n, cell* c, char ch);
-unsigned cell_bchannel(const cell* cl);
-unsigned cell_fchannel(const cell* cl);
-uint64_t cell_set_bchannel(cell* cl, uint32_t channel);
-uint64_t cell_set_fchannel(cell* cl, uint32_t channel);
-uint64_t cell_blend_fchannel(cell* cl, unsigned channel, unsigned* blends);
-uint64_t cell_blend_bchannel(cell* cl, unsigned channel, unsigned* blends);
-unsigned cell_fg(const cell* cl);
-unsigned cell_bg(const cell* cl);
-unsigned cell_fg_alpha(const cell* cl);
-unsigned cell_bg_alpha(const cell* cl);
-unsigned cell_fg_rgb(const cell* cl, unsigned* r, unsigned* g, unsigned* b);
-unsigned cell_bg_rgb(const cell* cl, unsigned* r, unsigned* g, unsigned* b);
-int cell_set_fg_rgb(cell* cl, int r, int g, int b);
-int cell_set_bg_rgb(cell* cl, int r, int g, int b);
-void cell_set_fg_rgb_clipped(cell* cl, int r, int g, int b);
-void cell_set_bg_rgb_clipped(cell* cl, int r, int g, int b);
-int cell_set_fg(cell* c, uint32_t channel);
-int cell_set_bg(cell* c, uint32_t channel);
-bool cell_fg_default_p(const cell* cl);
-bool cell_bg_default_p(const cell* cl);
 typedef struct palette256 {
   // We store the RGB values as a regular ol' channel
   uint32_t chans[256];
 } palette256;
 palette256* palette256_new(struct notcurses* nc);
 int palette256_use(struct notcurses* nc, const palette256* p);
-int palette256_set_rgb(palette256* p, int idx, int r, int g, int b);
-int palette256_set(palette256* p, int idx, unsigned rgb);
-int palette256_get_rgb(const palette256* p, int idx, unsigned* r, unsigned* g, unsigned* b);
 void palette256_free(palette256* p);
-typedef enum {
-  NCERR_SUCCESS,
-  NCERR_NOMEM,
-  NCERR_EOF,
-  NCERR_DECODE,
-  NCERR_UNIMPLEMENTED,
-} nc_err_e;
 typedef enum {
   NCSCALE_NONE,
   NCSCALE_SCALE,
@@ -247,13 +195,13 @@ typedef enum {
 } ncscale_e;
 int notcurses_lex_scalemode(const char* op, ncscale_e* scalemode);
 const char* notcurses_str_scalemode(ncscale_e scalemode);
-struct ncvisual* ncvisual_from_file(const char* file, nc_err_e* ncerr);
+struct ncvisual* ncvisual_from_file(const char* file);
 struct ncvisual* ncvisual_from_rgba(const void* rgba, int rows, int rowstride, int cols);
 struct ncvisual* ncvisual_from_bgra(const void* rgba, int rows, int rowstride, int cols);
 struct ncvisual* ncvisual_from_plane(const struct ncplane* n, ncblitter_e blit, int begy, int begx, int leny, int lenx);
 int ncvisual_geom(const struct notcurses* nc, const struct ncvisual* n, const struct ncvisual_options* vopts, int* y, int* x, int* toy, int* tox);
 void ncvisual_destroy(struct ncvisual* ncv);
-nc_err_e ncvisual_decode(struct ncvisual* nc);
+int ncvisual_decode(struct ncvisual* nc);
 int ncvisual_rotate(struct ncvisual* n, double rads);
 int ncvisual_resize(struct ncvisual* n, int rows, int cols);
 int ncvisual_polyfill_yx(struct ncvisual* n, int y, int x, uint32_t rgba);
@@ -262,7 +210,7 @@ char* ncvisual_subtitle(const struct ncvisual* ncv);
 int ncvisual_at_yx(const struct ncvisual* n, int y, int x, uint32_t* pixel);
 int ncvisual_set_yx(const struct ncvisual* n, int y, int x, uint32_t pixel);
 typedef int (*streamcb)(struct ncvisual*, struct ncvisual_options*, const struct timespec*, void*);
-int ncvisual_stream(struct notcurses* nc, struct ncvisual* ncv, nc_err_e* ncerr, float timescale, streamcb streamer, const struct ncvisual_options* vopts, void* curry);
+int ncvisual_stream(struct notcurses* nc, struct ncvisual* ncv, float timescale, streamcb streamer, const struct ncvisual_options* vopts, void* curry);
 struct ncvisual_options {
   struct ncplane* n;
   ncscale_e scaling;
@@ -383,25 +331,26 @@ int ncreel_destroy(struct ncreel* pr);
 void* nctablet_userptr(struct nctablet* t);
 struct ncplane* nctablet_ncplane(struct nctablet* t);
 int ncplane_polyfill_yx(struct ncplane* n, int y, int x, const cell* c);
-int ncplane_gradient(struct ncplane* n, const char* egc, uint32_t attrword, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ystop, int xstop);
-int ncplane_gradient_sized(struct ncplane* n, const char* egc, uint32_t attrword, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ylen, int xlen);
+int ncplane_gradient(struct ncplane* n, const char* egc, uint32_t styles, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ystop, int xstop);
 int ncplane_highgradient(struct ncplane* n, uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr, int ystop, int xstop);
-int ncplane_highgradient_sized(struct ncplane* n, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ylen, int xlen);
+int ncplane_highgradient_sized(struct ncplane* n, uint32_t ul, uint32_t ur, uint32_t ll, uint32_t lr, int ylen, int xlen);
 int ncplane_putsimple_stainable(struct ncplane* n, char c);
 int ncplane_putegc_stainable(struct ncplane* n, const char* gclust, int* sbytes);
 int ncplane_putwegc_stainable(struct ncplane* n, const wchar_t* gclust, int* sbytes);
-int ncplane_format(struct ncplane* n, int ystop, int xstop, uint32_t attrword);
+int ncplane_format(struct ncplane* n, int ystop, int xstop, uint32_t styles);
 int ncplane_stain(struct ncplane* n, int ystop, int xstop, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr);
 int ncplane_rotate_cw(struct ncplane* n);
 int ncplane_rotate_ccw(struct ncplane* n);
 void ncplane_translate(const struct ncplane* src, const struct ncplane* dst, int* y, int* x);
 bool ncplane_translate_abs(const struct ncplane* n, int* y, int* x);
 typedef struct ncplot_options {
-  uint64_t maxchannel;
-  uint64_t minchannel;
+  uint64_t maxchannels;
+  uint64_t minchannels;
+  uint16_t legendstyle;
   ncblitter_e gridtype;
   uint64_t rangex;
   unsigned flags;
+  const char* title;
 } ncplot_options;
 struct ncuplot* ncuplot_create(struct ncplane* n, const ncplot_options* opts, uint64_t miny, uint64_t maxy);
 struct ncdplot* ncdplot_create(struct ncplane* n, const ncplot_options* opts, double miny, double maxy);
@@ -475,6 +424,7 @@ int ncdirect_cursor_up(struct ncdirect* nc, int num);
 int ncdirect_cursor_left(struct ncdirect* nc, int num);
 int ncdirect_cursor_right(struct ncdirect* nc, int num);
 int ncdirect_cursor_down(struct ncdirect* nc, int num);
+int ncdirect_flush(const struct ncdirect* nc);
 int ncdirect_hline_interp(struct ncdirect* n, const char* egc, int len, uint64_t h1, uint64_t h2);
 int ncdirect_vline_interp(struct ncdirect* n, const char* egc, int len, uint64_t h1, uint64_t h2);
 int ncdirect_box(struct ncdirect* n, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, const wchar_t* wchars, int ylen, int xlen, unsigned ctlword);
@@ -482,9 +432,16 @@ int ncdirect_rounded_box(struct ncdirect* n, uint64_t ul, uint64_t ur, uint64_t
 int ncdirect_double_box(struct ncdirect* n, uint64_t ul, uint64_t ur, uint64_t ll, uint64_t lr, int ylen, int xlen, unsigned ctlword);
 bool ncdirect_canopen_images(const struct ncdirect* n);
 bool ncdirect_canutf8(const struct ncdirect* n);
-nc_err_e ncdirect_render_image(struct ncdirect* n, const char* filename, ncalign_e align, ncblitter_e blitter, ncscale_e scale);
+int ncdirect_render_image(struct ncdirect* n, const char* filename, ncalign_e align, ncblitter_e blitter, ncscale_e scale);
 struct ncplane* ncplane_parent(struct ncplane* n);
 const struct ncplane* ncplane_parent_const(const struct ncplane* n);
+int notcurses_cursor_enable(struct notcurses* nc, int y, int x);
+int notcurses_cursor_disable(struct notcurses* nc);
+int ncreader_move_left(struct ncreader* n);
+int ncreader_move_right(struct ncreader* n);
+int ncreader_move_up(struct ncreader* n);
+int ncreader_move_down(struct ncreader* n);
+int ncreader_write_egc(struct ncreader* n, const char* egc);
 """)
 
 if __name__ == "__main__":
